@@ -1,4 +1,5 @@
 #![doc = include_str!("../README.md")]
+#![deny(missing_docs, rustdoc::broken_intra_doc_links)]
 
 pub mod error;
 mod os_impl;
@@ -76,23 +77,39 @@ bitflags! {
 }
 
 /// The preferred size of the pages uses, where the size is in log2 notation.
+///
+/// Note that not all the offered page sizes may be available on the current platform.
 #[derive(Clone, Copy, Debug, Eq, Ord, PartialEq, PartialOrd)]
 pub struct PageSize(pub usize);
 
 impl PageSize {
+    /// Map the mapping using 4 KiB pages.
     pub const _4K:   Self = Self(12);
+    /// Map the mapping using 64 KiB pages.
     pub const _64K:  Self = Self(16);
+    /// Map the mapping using 512 KiB pages.
     pub const _512K: Self = Self(19);
+    /// Map the mapping using 1 MiB pages.
     pub const _1M:   Self = Self(20);
+    /// Map the mapping using 2 MiB pages.
     pub const _2M:   Self = Self(21);
+    /// Map the mapping using 4 MiB pages.
     pub const _4M:   Self = Self(22);
+    /// Map the mapping using 8 MiB pages.
     pub const _8M:   Self = Self(23);
+    /// Map the mapping using 16 MiB pages.
     pub const _16M:  Self = Self(24);
+    /// Map the mapping using 32 MiB pages.
     pub const _32M:  Self = Self(25);
+    /// Map the mapping using 256 MiB pages.
     pub const _256M: Self = Self(28);
+    /// Map the mapping using 512 MiB pages.
     pub const _512M: Self = Self(29);
+    /// Map the mapping using 1 GiB pages.
     pub const _1G:   Self = Self(30);
+    /// Map the mapping using 2 GiB pages.
     pub const _2G:   Self = Self(31);
+    /// Map the mapping using 16 GiB pages.
     pub const _16G:  Self = Self(34);
 }
 
@@ -152,6 +169,8 @@ macro_rules! mmap_impl {
             }
 
             /// Remaps this memory mapping as inaccessible.
+            ///
+            /// In case of failure, this returns the ownership of `self`.
             pub fn make_none(self) -> Result<MmapNone, (Self, Error)> {
                 if let Err(e) = self.inner.make_none() {
                     return Err((self, e));
@@ -163,6 +182,8 @@ macro_rules! mmap_impl {
             }
 
             /// Remaps this memory mapping as immutable.
+            ///
+            /// In case of failure, this returns the ownership of `self`.
             pub fn make_read_only(self) -> Result<Mmap, (Self, Error)> {
                 if let Err(e) = self.inner.make_read_only() {
                     return Err((self, e));
@@ -174,6 +195,8 @@ macro_rules! mmap_impl {
             }
 
             /// Remaps this memory mapping as executable.
+            ///
+            /// In case of failure, this returns the ownership of `self`.
             pub fn make_exec(self) -> Result<Mmap, (Self, Error)> {
                 if let Err(e) = self.inner.make_exec() {
                     return Err((self, e));
@@ -196,6 +219,8 @@ macro_rules! mmap_impl {
             /// modified the pages, then executing the code may result in undefined behavior. To ensure
             /// correct behavior a user has to flush the instruction cache after modifying and before
             /// executing the page.
+            ///
+            /// In case of failure, this returns the ownership of `self`.
             pub unsafe fn make_exec_no_flush(self) -> Result<Mmap, (Self, Error)> {
                 if let Err(e) = self.inner.make_exec() {
                     return Err((self, e));
@@ -271,6 +296,16 @@ pub struct Mmap {
 
 mmap_impl!(Mmap);
 
+impl Mmap {
+    /// Extracts a slice containing the entire mapping.
+    ///
+    /// This is equivalent to `&mapping[..]`.
+    #[inline]
+    pub fn as_slice(&self) -> &[u8] {
+        &self[..]
+    }
+}
+
 impl Deref for Mmap {
     type Target = [u8];
 
@@ -297,6 +332,22 @@ pub struct MmapMut {
 mmap_impl!(MmapMut);
 
 impl MmapMut {
+    /// Extracts a slice containing the entire mapping.
+    ///
+    /// This is equivalent to `&mapping[..]`.
+    #[inline]
+    pub fn as_slice(&self) -> &[u8] {
+        &self[..]
+    }
+
+    /// Extracts a mutable slice containing the entire mapping.
+    ///
+    /// This is equivalent to `&mut mapping[..]`.
+    #[inline]
+    pub fn as_mut_slice(&mut self) -> &mut [u8] {
+        &mut self[..]
+    }
+
     /// Yields a raw mutable pointer to this mapping.
     #[inline]
     pub fn as_mut_ptr(&mut self) -> *mut u8 {
@@ -344,10 +395,11 @@ pub struct MmapOptions {
 }
 
 impl MmapOptions {
-    /// Construct the `MmapOptions` builder.
-    pub fn new() -> Self {
+    /// Constructs the `MmapOptions` builder. The size specified is the size of the mapping to be
+    /// allocated in bytes.
+    pub fn new(size: usize) -> Self {
         Self {
-            inner: platform::MmapOptions::new(),
+            inner: platform::MmapOptions::new(size),
         }
     }
 
@@ -360,7 +412,7 @@ impl MmapOptions {
     }
 
     /// The desired address at which the memory should be mapped.
-    pub fn with_address(self, address: Option<usize>) -> Self {
+    pub fn with_address(self, address: usize) -> Self {
         Self {
             inner: self.inner.with_address(address),
         }
@@ -375,30 +427,30 @@ impl MmapOptions {
     /// access, then [`Mmap::make_mut()`] will not work. Furthermore, [`std::fs::OpenOptions`] does
     /// not in itself provide a standardized way to open the file with executable access. However,
     /// if the file is not opened with executable access, then it may not be possible to use
-    /// [`Mmap::make_exec()`]. Fortunately, Rust provides [`std::os::windows::fs::OpenOptionsExt`]
-    /// that allows you to open the file with executable access rights. See
-    /// [`std::os::windows::fs::OpenOptionsExt::access_mode`] for more information.
+    /// [`Mmap::make_exec()`]. Fortunately, Rust provides [`OpenOptionsExt`] that allows you to
+    /// open the file with executable access rights. See [`access_mode`] for more information.
     ///
     /// This function is marked as **unsafe** as the user should be aware that even in the case
     /// that a file is mapped as immutable in the address space of the current process, it does not
     /// guarantee that there does not exist any other mutable mapping to the file.
     ///
     /// On Microsoft Windows, it is possible to limit the access to shared reading or to be fully
-    /// exclusive using [`std::os::windows::fs::OpenOptionsExt::share_mode`].
+    /// exclusive using [`share_mode`].
     ///
     /// On most Unix systems, it is possible to use [`nix::fcntl::flock`]. However, keep in mind
     /// that this provides an **advisory** locking scheme, and that implementations are therefore
     /// required to be co-operative.
-    pub unsafe fn with_file(self, file: Option<(File, u64)>) -> Self {
+    ///
+    /// On Linux, it is also possible to mark the file as immutable. See `man 2 ioctl_iflags` and
+    /// `man 1 chattr` for more information.
+    ///
+    /// [`OpenOptionsExt`]: https://doc.rust-lang.org/std/os/windows/fs/trait.OpenOptionsExt.html
+    /// [`access_mode`]: https://doc.rust-lang.org/std/os/windows/fs/trait.OpenOptionsExt.html#tymethod.access_mode
+    /// [`share_mode`]: https://doc.rust-lang.org/std/os/windows/fs/trait.OpenOptionsExt.html#tymethod.share_mode
+    /// [`nix::fcntl::flock`]: https://docs.rs/nix/latest/nix/fcntl/fn.flock.html
+    pub unsafe fn with_file(self, file: File, offset: u64) -> Self {
         Self {
-            inner: self.inner.with_file(file),
-        }
-    }
-
-    /// The size of the mapping to be allocated.
-    pub fn with_size(self, size: usize) -> Self {
-        Self {
-            inner: self.inner.with_size(size),
+            inner: self.inner.with_file(file, offset),
         }
     }
 
@@ -420,7 +472,7 @@ impl MmapOptions {
     }
 
     /// Whether this memory mapped should be backed by a specific page size or not.
-    pub fn with_page_size(self, page_size: Option<PageSize>) -> Self {
+    pub fn with_page_size(self, page_size: PageSize) -> Self {
         Self {
             inner: self.inner.with_page_size(page_size),
         }
